/// <reference types="node" />
import { TSClone, TSObject } from "./tsobject";
import { Comparison, uint8 } from "./types";
/**
 * TSData is a mutable buffer class.
 */
export interface TSDataOptions {
    dontCopySourceBuffer?: boolean;
    fillWithZeros?: boolean;
    allocMethod?: (n: number) => Buffer;
}
export declare class TSData implements Iterable<number>, TSObject, TSClone<TSData> {
    protected _len: number;
    protected _buf: Buffer;
    private _allocFn;
    constructor(source?: TSData | Buffer | number | null | undefined, opts?: TSDataOptions);
    [Symbol.iterator](): {
        next: () => {
            done: boolean;
            value: number;
        };
    };
    clone(): TSData;
    appendData(source: TSData | Buffer | null | undefined, start?: number, end?: number): void;
    setData(source: TSData | Buffer | null | undefined, targetStart?: number, sourceStart?: number, sourceEnd?: number): void;
    appendByte(source: uint8): void;
    appendBytes(source: uint8[] | Uint8Array | null | undefined, start?: number, end?: number): void;
    appendAsciiString(source: string | null | undefined): void;
    replaceBytes(source: uint8[] | Uint8Array | null | undefined, targetStart?: number, sourceStart?: number, sourceEnd?: number): void;
    replaceAsciiString(source: string | null | undefined, targetStart?: number): void;
    get capacity(): number;
    get length(): number;
    set length(n: number);
    get mutableBuffer(): Buffer;
    get buffer(): Buffer;
    get internalStorage(): [Buffer, number];
    entries(): IterableIterator<[number, number]>;
    keys(): IterableIterator<number>;
    values(): IterableIterator<number>;
    includes(value: Buffer | TSData | number | null | undefined, byteOffset?: number): boolean;
    indexOf(value: TSData | number | Uint8Array | null | undefined, byteOffset?: number): number;
    lastIndexOf(value: TSData | number | Uint8Array | null | undefined, byteOffset?: number): number;
    slice(begin?: number, end?: number): TSData;
    copy(targetBuffer: Uint8Array | TSData, targetStart?: number, sourceStart?: number, sourceEnd?: number): number;
    writeToFile(path: string): boolean;
    equals(otherBuffer: Uint8Array): boolean;
    toString(encoding?: ((b: Buffer, start: number, end: number) => string) | BufferEncoding, start?: number, end?: number): string;
    toJSON(): any;
    toArray(): number[];
    compare(other: any): Comparison;
    isEqual(other: any): boolean;
    readBigUInt64BE(offset?: number): bigint;
    readBigUInt64LE(offset?: number): bigint;
    readBigInt64BE(offset?: number): bigint;
    readBigInt64LE(offset?: number): bigint;
    readUInt8(offset?: number): number;
    readUInt16LE(offset?: number): number;
    readUInt16BE(offset?: number): number;
    readUInt32LE(offset?: number): number;
    readUInt32BE(offset?: number): number;
    readInt8(offset?: number): number;
    readInt16LE(offset?: number): number;
    readInt16BE(offset?: number): number;
    readInt32LE(offset?: number): number;
    readInt32BE(offset?: number): number;
    readFloatLE(offset?: number): number;
    readFloatBE(offset?: number): number;
    readDoubleLE(offset?: number): number;
    readDoubleBE(offset?: number): number;
    writeBigInt64BE(value: bigint, offset?: number): void;
    writeBigInt64LE(value: bigint, offset?: number): void;
    writeBigUInt64BE(value: bigint, offset?: number): void;
    writeBigUInt64LE(value: bigint, offset?: number): void;
    writeUInt8(value: number, offset?: number): void;
    writeUInt16LE(value: number, offset?: number): void;
    writeUInt16BE(value: number, offset?: number): void;
    writeUInt32LE(value: number, offset?: number): void;
    writeUInt32BE(value: number, offset?: number): void;
    writeInt8(value: number, offset?: number): void;
    writeInt16LE(value: number, offset?: number): void;
    writeInt16BE(value: number, offset?: number): void;
    writeInt32LE(value: number, offset?: number): void;
    writeInt32BE(value: number, offset?: number): void;
    writeFloatLE(value: number, offset?: number): void;
    writeFloatBE(value: number, offset?: number): void;
    writeDoubleLE(value: number, offset?: number): void;
    writeDoubleBE(value: number, offset?: number): void;
    protected _willGrow(n: number): void;
    protected _copyTo(target: Buffer): void;
    protected _read<T>(offset: number | undefined, size: number, bufferReadFn: (offset?: number) => T): T;
    protected _write<T>(value: T, offset: number | undefined, size: number, bufferWriteFn: (value: T, offset?: number) => void): void;
}
export interface TSDataConstructor {
    new (source?: TSData | Buffer | number | null | undefined, opts?: TSDataOptions): TSData;
}
