/// <reference types="node" />
import { TSData, TSDataOptions } from "./tsdata";
export declare type TSEndianness = 'BE' | 'LE';
export interface TSDataCursorOptions extends TSDataOptions {
    cursor?: number;
    endianness?: TSEndianness;
}
export declare class TSDataCursor extends TSData {
    protected _cursor: number;
    protected _endianness: TSEndianness;
    constructor(source?: TSData | Buffer | number | null | undefined, opts?: TSDataCursorOptions);
    get bigEndian(): boolean;
    get littleEndian(): boolean;
    get endianness(): TSEndianness;
    get cursor(): number;
    set cursor(n: number);
    readBigUInt64(offset?: number): bigint;
    readBigInt64(offset?: number): bigint;
    readUInt16(offset?: number): number;
    readUInt32(offset?: number): number;
    readInt16(offset?: number): number;
    readInt32(offset?: number): number;
    readFloat(offset?: number): number;
    readDouble(offset?: number): number;
    writeBigInt64(value: bigint, offset?: number): void;
    writeBigUInt64(value: bigint, offset?: number): void;
    writeUInt16(value: number, offset?: number): void;
    writeUInt32(value: number, offset?: number): void;
    writeInt16(value: number, offset?: number): void;
    writeInt32(value: number, offset?: number): void;
    writeFloat(value: number, offset?: number): void;
    writeDouble(value: number, offset?: number): void;
    protected _read<T>(offset: number | undefined, size: number, bufferReadFn: (offset?: number) => T): T;
    protected _write<T>(value: T, offset: number | undefined, size: number, bufferWriteFn: (value: T, offset?: number) => void): void;
}
export interface TSDataCursorConstructor {
    new (source?: TSData | Buffer | number | null | undefined, opts?: TSDataCursorOptions): TSDataCursor;
}
