import { TSClone, TSObject } from "./tsobject";
import { Comparison } from "./types";
export interface TSPoint {
    x: number;
    y: number;
}
export interface TSSize {
    w: number;
    h: number;
}
export declare enum TSRectEdge {
    TSMinXEdge = 0,
    TSMinYEdge = 1,
    TSMaxXEdge = 2,
    TSMaxYEdge = 3
}
export declare type TSDocumentFormat = 'letter' | 'legal' | 'tabloid' | 'a0' | 'a1' | 'a2' | 'a3' | 'a4' | 'a5' | 'a6';
export declare const TSDocumentFormats: {
    [key in TSDocumentFormat]: TSSize;
};
export declare class TSRect implements TSPoint, TSSize, TSObject, TSClone<TSRect> {
    x: number;
    y: number;
    w: number;
    h: number;
    constructor();
    constructor(rect: TSRect);
    constructor(format: TSDocumentFormat);
    constructor(arrayRect: number[]);
    constructor(x: number, y: number, w: number, h: number);
    constructor(origin: TSPoint, w: number, h: number);
    constructor(x: number, y: number, size: TSSize | TSDocumentFormat);
    constructor(origin: TSPoint, size: TSSize | TSDocumentFormat);
    get origin(): TSPoint;
    get size(): TSSize;
    get width(): number;
    get height(): number;
    get minX(): number;
    get minY(): number;
    get maxX(): number;
    get maxY(): number;
    get midX(): number;
    get midY(): number;
    get isEmpty(): boolean;
    contains(p: TSPoint | TSRect | number[] | null | undefined): boolean;
    containsPoint(p: TSPoint | number[] | null | undefined): boolean;
    containsRect(p: TSRect | number[] | null | undefined): boolean;
    containedIn(r: TSRect | number[] | null | undefined): boolean;
    containedInRect(r: TSRect | number[] | null | undefined): boolean;
    intersects(r: TSRect | number[] | null | undefined): boolean;
    intersectsRect(r: TSRect | number[] | null | undefined): boolean;
    intersection(r: TSRect | number[] | null | undefined): TSRect;
    intersectionRect(r: TSRect | number[] | null | undefined): TSRect;
    union(r: TSRect | number[] | undefined | null): TSRect;
    unionRect(r: TSRect | number[] | null | undefined): TSRect;
    offset(dx: number, dy: number): TSRect;
    offsetRect(dx: number, dy: number): TSRect;
    inset(dw: number, dh: number): TSRect;
    insetRect(dw: number, dh: number): TSRect;
    integral(): TSRect;
    integralRect(): TSRect;
    /**
     *
     * @param amount the part of widh or height we keep for the slice
     * @param edge on which edge do we decide to divide our rect
     * @returns a tupple with first the slice we want and second the remainding part of the rect
     */
    divide(amount: number, edge: TSRectEdge): [TSRect, TSRect];
    divideRect(amount: number, edge: TSRectEdge): [TSRect, TSRect];
    clone(): TSRect;
    isEqual(other: any): boolean;
    compare(other: any): Comparison;
    toJSON(): object;
    toString(): string;
    toArray(): number[];
}
export declare function TSmm2Pixels(mm: number): number;
export declare function TScm2Pixels(cm: number): number;
export declare function TSPixels2cm(pixels: number): number;
export declare function TSPixels2mm(pixels: number): number;
export declare function TSInches2Pixels(inches: number): number;
export declare function TSPixels2Inches(pixels: number): number;
export declare function TSEqualSizes(A: TSSize, B: TSSize): boolean;
export declare function TSEqualPoints(A: TSPoint, B: TSPoint): boolean;
