import { TSRange } from "./tsrange";
import { AnyDictionary } from "./types";
import { TSCollection } from "./tsobject";
export declare type QualifierOperator = 'AND' | 'OR' | 'NOT' | 'EQ' | 'NEQ' | 'LT' | 'LTE' | 'GT' | 'GTE' | 'LIKE' | 'IN' | 'NIN' | 'OK' | 'KO';
export declare type QualifierOperand = any;
export declare type QualifierCondition<T> = AnyDictionary | TSQualifier<T>;
export declare const LAZY_INTERSECTION = false;
export declare type TSQualifierCollection<T> = ArrayLike<T> | Set<T> | TSCollection<T>;
export declare type RecursiveKeyPath<T, K extends keyof T> = K extends string ? T[K] extends Record<string, any> ? T[K] extends TSQualifierCollection<infer U> ? K | `${K}.${RecursiveKeyPath<Required<U>, keyof Required<U>>}` : K | `${K}.${RecursiveKeyPath<Required<T[K]>, keyof Required<T[K]>>}` : K : never;
export declare type KeyPath<T> = RecursiveKeyPath<Required<T>, keyof Required<T>> | keyof Required<T>;
export declare class TSQualifier<T> {
    operator: QualifierOperator;
    operands: QualifierOperand[];
    protected constructor(operator: QualifierOperator, operands: QualifierOperand[]);
    static OR<T>(conds?: QualifierCondition<T>[]): TSQualifier<T>;
    static AND<T>(conds?: QualifierCondition<T>[]): TSQualifier<T>;
    static NOT<T>(cond: QualifierCondition<T>): TSQualifier<T>;
    static OK<T>(key: KeyPath<T>): TSQualifier<T>;
    static KO<T>(key: KeyPath<T>): TSQualifier<T>;
    static EQ<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static NEQ<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static LT<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static LTE<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static GT<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static GTE<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static LIKE<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static IN<T>(key: KeyPath<T>, values: QualifierOperand[]): TSQualifier<T>;
    static NIN<T>(key: KeyPath<T>, values: QualifierOperand[]): TSQualifier<T>;
    static INCLUDES<T>(key1: KeyPath<T>, key2: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static INCLUDED<T>(key: KeyPath<T>, value1: QualifierOperand, value2: QualifierOperand, canUnspecify?: boolean, strict?: boolean): TSQualifier<T>;
    static INTERSECTS<T>(key1: KeyPath<T>, key2: KeyPath<T>, value1: QualifierOperand, value2: QualifierOperand, canUnspecify?: boolean, strict?: boolean): TSQualifier<T>;
    static INRANGE<T>(key: KeyPath<T>, value: TSRange): TSQualifier<T>;
    static intersectionQualifiers<T>(A: QualifierOperand, B: QualifierOperand, C: KeyPath<T>, D: KeyPath<T>, strict: boolean, canUnspecify: boolean): Array<TSQualifier<T>>;
    add(cond: QualifierCondition<T>): void;
    addValue(key: KeyPath<T>, value: QualifierOperand): void;
    addNotValue(key: KeyPath<T>, value: QualifierOperand): void;
    addRange(key: KeyPath<T>, range: TSRange): void;
    addInclusion(key: KeyPath<T>, value1: QualifierOperand, value2: QualifierOperand, canUnspecify?: boolean, strict?: boolean): void;
    addIntersection(key1: KeyPath<T>, key2: KeyPath<T>, value1: QualifierOperand, value2: QualifierOperand, canUnspecify?: boolean, strict?: boolean): void;
    inverse(): TSQualifier<T>;
    request(): any;
    validateValueForCondition(v: T, condition: AnyDictionary): boolean;
    validateValue(v: T): boolean;
    filterValues(values: Iterable<T> | null | undefined): Array<T>;
}
export declare function $valuesForKeyPath<T>(v: any, path: KeyPath<T>): any;
declare global {
    export interface String {
        sqlLike: (this: string, like: string) => boolean;
    }
    export interface Array<T> {
        filterWithQualifier: <T>(qual: TSQualifier<T>) => Array<T>;
    }
}
export declare function $sqllike(a: any, b: string): any;
