export declare type groupFN = (t: TSTestGroup) => Promise<void>;
export declare type unaryFN = (t: TSUnaryTest) => Promise<void>;
export declare type testFN = (t: TSTest) => Promise<void>;
export declare class TSTester {
    groups: TSTestGroup[];
    desc: string;
    constructor(s: string);
    addGroup(grp: TSTestGroup): void;
    addGroups(grps: TSTestGroup | TSTestGroup[]): void;
    log(format: string, ...args: any[]): void;
    run(): Promise<void>;
}
export declare class TSTest {
    desc: string;
    fn: testFN;
    protected constructor(s: string, f: testFN);
    static group(s: string, f: groupFN): TSTestGroup;
}
export declare class TSTestGroup extends TSTest {
    private _unaries;
    constructor(s: string, f: groupFN);
    unary(s: string, f: unaryFN): void;
    prepare(): Promise<void>;
    log(format: string, ...args: any[]): void;
    run(): Promise<boolean>;
}
export declare class TSUnaryTest extends TSTest {
    _group: TSTestGroup;
    private _failed;
    private _passed;
    private _expected;
    constructor(g: TSTestGroup, s: string, f: unaryFN);
    log(format: string, ...args: any[]): void;
    run(): Promise<boolean>;
    expect(v: any, msg?: string): TSExpectAgent;
    fail(): void;
    pass(): void;
}
export declare class TSExpectAgent {
    private _value;
    private _step;
    private _message;
    constructor(step: TSUnaryTest, value: any, msg?: string);
    toBe(aValue: any): void;
    notToBe(aValue: any): void;
    toBeTruthy(): void;
    toBeFalsy(): void;
    toBeUndefined(): void;
    toBeDefined(): void;
    toBeNull(): void;
    toBeNotNull(): void;
    toBeOK(): void;
    toBeNotOK(): void;
    toBeNaN(): void;
    toBeANumber(): void;
    toBeAnInt(): void;
    toBeAnUnsigned(): void;
    toBeAString(): void;
    toBeBool(): void;
    toBeAnEmail(): void;
    toBeAnUrl(): void;
    toBeAnUUID(): void;
    toBeAnObject(): void;
    toBeArray(): void;
    toBeADate(): void;
    toBeAFunction(): void;
    gt(aValue: any): void;
    gte(aValue: any): void;
    lt(aValue: any): void;
    lte(aValue: any): void;
    private _compfail;
    private _elogfail;
    private _nelogfail;
    private _writeMessage;
}
