"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$stack = exports.$inbrowser = exports.$fatalerror = exports.$logheader = exports.$writeterm = exports.$logterm = exports.$termclean = exports.$term = exports.$timeout = void 0;
const commons_1 = require("./commons");
const process_1 = require("process");
function $timeout(promise, time, exception) {
    let timer;
    return Promise.race([
        promise,
        new Promise((_, rejection) => timer = setTimeout(rejection, time, exception))
    ]).finally(() => clearTimeout(timer));
}
exports.$timeout = $timeout;
function $term(s, escapeChar = '&') {
    let fmtlen = (0, commons_1.$length)(s);
    let ret = "";
    if (fmtlen) {
        let escape = false;
        if ((0, commons_1.$length)(escapeChar) !== 1 || escapeChar.includes('\x1b')) {
            escapeChar = '&';
        }
        for (let i = 0; i < fmtlen; i++) {
            const c = s.charAt(i);
            if (escape) {
                escape = false;
                switch (c) {
                    case escapeChar:
                        ret += escapeChar;
                        break;
                    case '0':
                        ret += "\x1b[0m";
                        break; // reset
                    // styles
                    case '1':
                        ret += "\x1b[1m";
                        break; // bright mode
                    case '>':
                        ret += "\x1b[2m";
                        break; // dimmed
                    case '/':
                        ret += "\x1b[3m";
                        break; // italic
                    case '_':
                        ret += "\x1b[4m";
                        break; // underscore
                    case '%':
                        ret += "\x1b[5m";
                        break; // blinked
                    case '<':
                        ret += "\x1b[7m";
                        break; // inversed
                    case '-':
                        ret += "\x1b[9m";
                        break; // strikethrough
                    // colors
                    case 'a':
                        ret += "\x1b[38;5;216m";
                        break; // apricot font
                    case 'A':
                        ret += "\x1b[48;5;216m";
                        break; // apricot background
                    case 'b':
                        ret += "\x1b[34m";
                        break; // blue font
                    case 'B':
                        ret += "\x1b[44m";
                        break; // blue background
                    case 'c':
                        ret += "\x1b[36m";
                        break; // cyan font
                    case 'C':
                        ret += "\x1b[46m";
                        break; // cyan background
                    case 'd':
                        ret += "\x1b[38;5;238m";
                        break; // dark gray font
                    case 'D':
                        ret += "\x1b[48;5;238m";
                        break; // dark gray background
                    case 'e':
                        ret += "\x1b[38;5;229m";
                        break; // egg white font
                    case 'E':
                        ret += "\x1b[48;5;229m";
                        break; // egg white background
                    // fF
                    case 'g':
                        ret += "\x1b[32m";
                        break; // green font  
                    case 'G':
                        ret += "\x1b[42m";
                        break; // green background
                    // hH
                    // iI
                    case 'j':
                        ret += "\x1b[38;5;121m";
                        break; // jungle green font
                    case 'J':
                        ret += "\x1b[48;5;121m";
                        break; // jungle green background
                    case 'k':
                        ret += "\x1b[30m";
                        break; // black font
                    case 'K':
                        ret += "\x1b[40m";
                        break; // black background
                    case 'l':
                        ret += "\x1b[38;5;252m";
                        break; // light gray font
                    case 'L':
                        ret += "\x1b[48;5;252m";
                        break; // light gray background
                    case 'm':
                        ret += "\x1b[35m";
                        break; // magenta font
                    case 'M':
                        ret += "\x1b[45m";
                        break; // magenta background
                    case 'o':
                        ret += "\x1b[38;5;208m";
                        break; // orange font
                    case 'O':
                        ret += "\x1b[48;5;208m";
                        break; // orange background
                    case 'p':
                        ret += "\x1b[38;5;212m";
                        break; // pink font
                    case 'P':
                        ret += "\x1b[48;5;212m";
                        break; // pink background
                    // qQ
                    case 'r':
                        ret += "\x1b[31m";
                        break; // red font
                    case 'R':
                        ret += "\x1b[41m";
                        break; // red background
                    // sS
                    // tT
                    case 'u':
                        ret += "\x1b[38;5;117m";
                        break; // uranian blue font
                    case 'U':
                        ret += "\x1b[48;5;117m";
                        break; // uranian blue background
                    case 'v':
                        ret += "\x1b[38;5;99m";
                        break; // violet font
                    case 'V':
                        ret += "\x1b[48;5;99m";
                        break; // violet background
                    case 'w':
                        ret += "\x1b[37m";
                        break; // white font 
                    case 'W':
                        ret += "\x1b[47m";
                        break; // white background
                    case 'x':
                        ret += "\x1b[38;5;244m";
                        break; // gray font
                    case 'X':
                        ret += "\x1b[48;5;244m";
                        break; // gray background
                    case 'y':
                        ret += "\x1b[33m";
                        break; // yellow font
                    case 'Y':
                        ret += "\x1b[43m";
                        break; // yellow background
                    // zZ
                    default:
                        ret += escapeChar;
                        ret += c;
                        break;
                }
            }
            else if (c === escapeChar) {
                escape = true;
            }
            else {
                ret += c;
            }
        }
        if (escape) {
            ret += escapeChar;
        }
    }
    return ret;
}
exports.$term = $term;
function $termclean(s, escapeChar = '&') {
    let len = (0, commons_1.$length)(s);
    let ret = "";
    if (len) {
        let State;
        (function (State) {
            State[State["Standard"] = 0] = "Standard";
            State[State["EscapeChar"] = 1] = "EscapeChar";
            State[State["EscapeEscape"] = 2] = "EscapeEscape";
        })(State || (State = {}));
        ;
        let state = State.Standard;
        if ((0, commons_1.$length)(escapeChar) !== 1 || escapeChar.includes('\x1b')) {
            escapeChar = '&';
        }
        for (let i = 0; i < len; i++) {
            const c = s.charAt(i);
            switch (state) {
                case State.Standard:
                    if (c === escapeChar) {
                        state = State.EscapeChar;
                    }
                    else if (c === '\x1b') {
                        state = State.EscapeEscape;
                    }
                    else {
                        ret += c;
                    }
                    break;
                case State.EscapeEscape:
                    if (c === 'm') {
                        state = State.Standard;
                    }
                    if (!"[0123456789;".includes(c)) {
                        state = State.Standard;
                        ret += c;
                    }
                    break;
                case State.EscapeChar:
                    if (c === escapeChar) {
                        ret += escapeChar;
                    }
                    else if (!"01>/_%<-aAbBcCdDeEgGjJkKlLmMoOpPrRuUvVwWxXyY".includes(c)) {
                        ret += escapeChar + c;
                    }
                    state = State.Standard;
            }
        }
    }
    return ret;
}
exports.$termclean = $termclean;
function $logterm(format, ...args) {
    format = $term((0, commons_1.$length)(format) ? format : '');
    if ((0, commons_1.$count)(args)) {
        format += args.join();
    }
    console.log(format);
}
exports.$logterm = $logterm;
function $writeterm(format, ...args) {
    format = $term((0, commons_1.$length)(format) ? format : '');
    if ((0, commons_1.$count)(args)) {
        format += args.join();
    }
    if (format.length) {
        process.stdout.write(format);
    }
}
exports.$writeterm = $writeterm;
function $logheader(s, width = 0, style = '&w', starStyle = '&x') {
    if (!width) {
        width = s.length;
    }
    $logterm("\n&0" + starStyle + "".padEnd(width + 4, '*'));
    $logterm(starStyle + '* ' + style + s.padEnd(width + 1, ' ') + '&0' + starStyle + '*');
    $logterm(starStyle + "".padEnd(width + 4, '*') + "&0\n");
}
exports.$logheader = $logheader;
function $fatalerror(test, s = 'FATAL ERROR', exitCode = -1, style = '&R&w') {
    if (test) {
        if ((0, commons_1.$ok)(exitCode)) {
            $logterm(style + ' ' + s + ` &0&o —— EXITING with code &O&k ${exitCode} &0\n`);
            (0, process_1.exit)(exitCode);
        }
        else {
            $logterm(style + ' ' + s + ` &0&o —— will throw`);
            throw s;
        }
    }
}
exports.$fatalerror = $fatalerror;
function $inbrowser() {
    // Check if the environment is Node.js
    if (typeof process === "object" &&
        typeof require === "function") {
        return false;
    }
    // Check if the environment is a Browser
    if (typeof window === "object") {
        return true;
    }
    return false;
}
exports.$inbrowser = $inbrowser;
function $stack() {
    const previousPrepareStackTrace = Error.prepareStackTrace;
    Error.prepareStackTrace = (_, stack) => stack;
    const stack = new Error().stack;
    Error.prepareStackTrace = previousPrepareStackTrace;
    return stack;
}
exports.$stack = $stack;
//# sourceMappingURL=utils.js.map